@@ -26,7 +26,10 @@ import os.path
 import re
 import sys
 import tempfile
-import unittest
+if sys.version_info[0] >= 3:
+    import unittest
+else:
+    import unittest2 as unittest
 import warnings
 
 import six
@@ -40,6 +43,7 @@ else:
 import apt_pkg   #type: ignore
 
 from debian import deb822
+from debian.debian_support import Version
 
 
 UNPARSED_PACKAGE = '''\
@@ -455,14 +459,38 @@ class TestDeb822(unittest.TestCase):
         for d in deb822.Deb822.iter_paragraphs(text):
             self.assertWellParsed(d, PARSED_PACKAGE)
 
-    def test_iter_paragraphs_file(self):
+    def test_iter_paragraphs_file_io(self):
         text = StringIO(UNPARSED_PACKAGE + '\n\n\n' + UNPARSED_PACKAGE)
 
         for d in deb822.Deb822.iter_paragraphs(text, use_apt_pkg=False):
             self.assertWellParsed(d, PARSED_PACKAGE)
 
-        for d in deb822.Deb822.iter_paragraphs(text, use_apt_pkg=True):
-            self.assertWellParsed(d, PARSED_PACKAGE)
+        with self.assertWarns(UserWarning):
+            # The StringIO is not a real file so this will raise a warning
+            for d in deb822.Deb822.iter_paragraphs(text, use_apt_pkg=True):
+                self.assertWellParsed(d, PARSED_PACKAGE)
+
+    def test_iter_paragraphs_file(self):
+        text = StringIO()
+        text.write(UNPARSED_PACKAGE)
+        text.write('\n\n\n')
+        text.write(UNPARSED_PACKAGE)
+
+        with tempfile.NamedTemporaryFile() as fh:
+            if sys.version_info[0] >= 3:
+                txt = text.getvalue().encode('UTF-8')
+            else:
+                txt = text.getvalue()
+            fh.write(txt)
+
+            fh.seek(0)
+            for d in deb822.Deb822.iter_paragraphs(fh, use_apt_pkg=False):
+                self.assertWellParsed(d, PARSED_PACKAGE)
+
+            fh.seek(0)
+            for d in deb822.Deb822.iter_paragraphs(fh, use_apt_pkg=True):
+                self.assertWellParsed(d, PARSED_PACKAGE)
+
 
     def test_iter_paragraphs_with_gpg(self):
         for string in GPG_SIGNED:
@@ -484,7 +512,7 @@ class TestDeb822(unittest.TestCase):
             self.assertWellParsed(d, PARSED_PACKAGE)
 
     def test_iter_paragraphs_with_extra_whitespace(self):
-        """ Paragraphs not elided when stray whitespace is between
+        """ Paragraphs splitting when stray whitespace is between
 
         From policy §5.1:
 
@@ -494,33 +522,59 @@ class TestDeb822(unittest.TestCase):
 
         On the principle of "be strict in what you send; be generous in
         what you receive", deb822 should permit such extra whitespace between
-        deb822 stanzas.
+        deb822 stanzas. See #715558 for further details.
 
-        See #715558 for further details.
+        However, when dealing with Packages and Sources files, the behaviour
+        of apt is to not split on whitespace-only lines, and so the internal
+        parser must be able to avoid doing so when dealing with these data.
+        See #913274 for further details.
         """
         for extra_space in (" ", "  ", "\t"):
             text = six.u(UNPARSED_PACKAGE) + '%s\n' % extra_space + \
                         six.u(UNPARSED_PACKAGE)
-            count = len(list(deb822.Deb822.iter_paragraphs(text)))
-            self.assertEqual(2, count,
-                        "Wrong number paragraphs were found in list: 2 != %d" % count)
 
             fd, filename = tempfile.mkstemp()
             fp = os.fdopen(fd, 'wb')
             fp.write(text.encode('utf-8'))
             fp.close()
 
+            def test_count(cmd, expected, *args, **kwargs):
+                #print("\n", cmd.__class__, expected)
+                with open_utf8(filename) as fh:
+                    count = len(list(cmd(fh, *args, **kwargs)))
+                    self.assertEqual(
+                        expected,
+                        count,
+                        "Wrong number paragraphs were found: expected {expected}, got {count}".format(
+                            count=count,
+                            expected=expected,
+                        )
+                    )
+
             try:
-                with open_utf8(filename) as fh:
-                    count = len(list(deb822.Deb822.iter_paragraphs(fh)))
-                    self.assertEqual(2, count,
-                            "Wrong number paragraphs were found in file: 2 != %d" % count)
-                with open_utf8(filename) as fh:
-                    count = len(list(deb822.Packages.iter_paragraphs(fh)))
-                    # this time the apt_pkg parser should be used and this
-                    # *should* elide the paragraphs and make a mess
-                    self.assertEqual(count, 1,
-                            "Wrong number paragraphs were found in file: 1 != %d" % count)
+                # apt_pkg not used, should split
+                test_count(deb822.Deb822.iter_paragraphs, 2)
+                test_count(deb822.Deb822.iter_paragraphs, 2, use_apt_pkg=False)
+
+                # apt_pkg used, should not split
+                test_count(deb822.Deb822.iter_paragraphs, 1, use_apt_pkg=True)
+
+                # Specialised iter_paragraphs force use of apt_pkg and don't split
+                test_count(deb822.Packages.iter_paragraphs, 1, use_apt_pkg=True)
+                test_count(deb822.Sources.iter_paragraphs, 1, use_apt_pkg=True)
+                test_count(deb822.Packages.iter_paragraphs, 1, use_apt_pkg=False)
+                test_count(deb822.Sources.iter_paragraphs, 1, use_apt_pkg=False)
+
+                # Explicitly set internal parser to not split
+                strict = {'whitespace-separates-paragraphs': False}
+                test_count(deb822.Packages.iter_paragraphs, 1, use_apt_pkg=False, strict=strict)
+                test_count(deb822.Sources.iter_paragraphs, 1, use_apt_pkg=False, strict=strict)
+
+                # Explicitly set internal parser to split
+                strict = {'whitespace-separates-paragraphs': True}
+                test_count(deb822.Packages.iter_paragraphs, 2, use_apt_pkg=False, strict=strict)
+                test_count(deb822.Sources.iter_paragraphs, 2, use_apt_pkg=False, strict=strict)
+
             finally:
                 os.remove(filename)
 
@@ -1044,6 +1098,7 @@ class TestPkgRelations(unittest.TestCase):
         pkgs = deb822.Packages.iter_paragraphs(f)
         pkg1 = next(pkgs)
         rel1 = {'breaks': [],
+                'built-using': [],
                 'conflicts': [],
                 'depends': [
                         [rel({'name': 'file', 'archqual': 'i386'})],
@@ -1077,6 +1132,7 @@ class TestPkgRelations(unittest.TestCase):
         self.assertPkgDictEqual(rel1, pkg1.relations)
         pkg2 = next(pkgs)
         rel2 = {'breaks': [],
+                'built-using': [],
                 'conflicts': [],
                 'depends': [
                         [rel({'name': 'lrzsz'})],
@@ -1238,6 +1294,35 @@ class TestPkgRelations(unittest.TestCase):
         self.assertEqual(deb822.PkgRelation.str(rel), r)
 
 
+class TestVersionAccessor(unittest.TestCase):
+
+    def test_get_version(self):
+        # should not be available in most basic Deb822
+        p = deb822.Deb822(UNPARSED_PACKAGE.splitlines())
+        with self.assertRaises(AttributeError):
+            p.get_version()
+
+        # should be available in Packages
+        p = deb822.Packages(UNPARSED_PACKAGE.splitlines())
+        v = p.get_version()
+        self.assertEqual(str(v), '1.5.12-1')
+        self.assertTrue(isinstance(v, Version))
+
+    def test_set_version(self):
+        # should not be available in most basic Deb822
+        p = deb822.Deb822(UNPARSED_PACKAGE.splitlines())
+        with self.assertRaises(AttributeError):
+            p.set_version()
+
+        # should be available in Packages
+        p = deb822.Packages(UNPARSED_PACKAGE.splitlines())
+        newver = '9.8.7-1'
+        v = Version(newver)
+        p.set_version(v)
+        self.assertEqual(p['Version'], newver)
+        self.assertTrue(isinstance(p['Version'], six.string_types))
+
+
 @unittest.skipUnless(os.path.exists('/usr/bin/gpgv'), "gpgv not installed")
 class TestGpgInfo(unittest.TestCase):
 
